import 'dart:convert';
import 'package:unittest/unittest.dart';
import 'package:unittest/html_config.dart';
import 'package:simple_exportable/simple_exportable.dart';

@Exportable()
class FlatClass extends Exportable {
	int id;
	String name;
}

@Exportable()
class LayerdClass extends Exportable {
	FlatClass flatClass;
	String name;
}

@Exportable()
class LayerdClasses extends Exportable {
	List<FlatClass> flatClasses;
	String name;
}

@Exportable()
class FlatClassExtra extends FlatClass {
	int extraId;
}

@Exportable()
class ClassWithSuperClass extends FlatClass{

}

void main() {
	useHtmlConfiguration();

	test('for flat object fills properties', () {
		var json = '''{"id":1,"name":"John"}''';
		FlatClass result = Exportable.getInstatiatedTypeForJson(FlatClass, json);

		expect(result.id, 1);
		expect(result.name, "John");
	});

	test('for list of flat objects fills properties', () {
		var json = '''[{"id":1,"name":"John"},{"id":2,"name":"Fred"}]''';
		List<FlatClass> result = Exportable.getInstatiatedTypeForJson(FlatClass, json);

		expect(result[0].id, 1);
		expect(result[0].name, "John");

		expect(result[1].id, 2);
		expect(result[1].name, "Fred");
	});

	test('for object with sub class fills properties', () {
		var json = '''{"flatClass": {"id":1,"name":"John"}, "name": "John"}''';
		LayerdClass result = Exportable.getInstatiatedTypeForJson(LayerdClass, json);

		expect(result.name, "John");

		expect(result.flatClass.id, 1);
		expect(result.flatClass.name, "John");
	});

	test('for object with list of sub class fills properties', () {
		var json = '''{"flatClasses": [{"id":1,"name":"John"},{"id":2,"name":"Fred"}], "name": "John"}''';
		LayerdClasses result = Exportable.getInstatiatedTypeForJson(LayerdClasses, json);

		expect(result.name, "John");

		expect(result.flatClasses[0].id, 1);
		expect(result.flatClasses[0].name, "John");

		expect(result.flatClasses[1].id, 2);
		expect(result.flatClasses[1].name, "Fred");
	});

	test('for object with null sub object filles properties', () {
		var json = '''{"flatClass": null, "name": "the name"}''';
		LayerdClass result = Exportable.getInstatiatedTypeForJson(LayerdClass, json);

		expect(result.name, "the name");

		expect(result.flatClass, null);
	});













	test('for flat object can convert to json', () {
		var flatClass = new FlatClass()
				..id = 1
				..name = "Name";

		var json = Exportable.getJsonFromObject(flatClass);

		expect(json, '''{"id":1,"name":"Name"}''');
	});

	test('for list of flat objects can convert to json', () {
		var flatClass = [new FlatClass()
					..id = 1
					..name = "Name", new FlatClass()
					..id = 2
					..name = "Name"];

		var json = Exportable.getJsonFromObject(flatClass);

		expect(json, '''[{"id":1,"name":"Name"},{"id":2,"name":"Name"}]''');
	});

	test('for object with sub class can convert to json', () {
		var layerdClass = new LayerdClass()
				..flatClass = (new FlatClass()
						..id = 1
						..name = "Name")
				..name = "Name";

		var json = Exportable.getJsonFromObject(layerdClass);

		expect(json, '''{"flatClass":{"id":1,"name":"Name"},"name":"Name"}''');
	});

	test('for object with list of sub class can convert to json', () {
		var layerdClass = new LayerdClasses()
				..flatClasses = [(new FlatClass()
							..id = 1
							..name = "Name"), (new FlatClass()
							..id = 2
							..name = "Name")]
				..name = "Name";

		var json = Exportable.getJsonFromObject(layerdClass);

		expect(json, '''{"flatClasses":[{"id":1,"name":"Name"},{"id":2,"name":"Name"}],"name":"Name"}''');
	});

	test('for object with superclass can convert to json', () {
    		var flatClass = new ClassWithSuperClass()
    				..id = 1
    				..name = "Name";

    		var json = Exportable.getJsonFromObject(flatClass);

    		expect(json, '''{"id":1,"name":"Name"}''');
	});

	test('for object with generic type can convert to json', (){
		var genericClass = new TheGenericClass<TheFieldClass>()..field = (new TheFieldClass()..name = "The Name");
		var json = Exportable.getJsonFromObject(genericClass);
		expect(json,'''{"field":{"name":"The Name"}}''');
	});
}

@Exportable()
class TheGenericClass<T> extends Exportable{
	T field;
}

@Exportable()
class TheFieldClass extends Exportable{
	String name;
}


